package JRoff;

public class RoffTriangle {

    private int p1;
    private int p2;
    private int p3;
    private double distance;
    private RoffVector viewVector;
    private RoffVector normal;
    private RoffVector centroid;
    private RoffVector[] pointArray;

    public RoffTriangle(int point1, int point2, int point3, RoffVector[] newPointArray) {
        p1 = point1;
        p2 = point2;
        p3 = point3;
        pointArray = newPointArray;
        buildCentroid();
        buildNormal();
    }

    public int getP1() {
        return p1;
    }
	
    public int getP2() {
        return p2;
    }

    public int getP3() {
        return p3;
    }

    public RoffVector getView() {
        return new RoffVector(viewVector.x , viewVector.y, viewVector.z);
    }

    RoffVector getNormal() {
        return new RoffVector(normal.x, normal.y, normal.z);
    }

    public RoffVector getCentroid() {
        return new RoffVector(centroid.x, centroid.y, centroid.z);
    }

    public void rebuild() {
	buildCentroid();
	buildNormal();
    }

    public void updateView(RoffVector fromPoint, RoffVector LoS) {

        viewVector = fromPoint.sub(centroid);
        distance = (centroid.sub(fromPoint)).dot(LoS);
        viewVector.normalize();
    }

    public boolean closer(RoffTriangle rhs) {
        return distance < rhs.distance;
    }

    public boolean sameDepth(RoffTriangle rhs) {
        return distance == rhs.distance;
    }

    public double getDepth() {
        return distance;
    }

    private void buildCentroid() {
	centroid = (
                    ( (pointArray[p1]).add(pointArray[p2])
                    ).add(pointArray[p3])
                   ).scalarDivide(3.0);
    }

    private void buildNormal() {
        normal = (
                  (pointArray[p2]).sub(pointArray[p1])
                 ).cross(
                  (pointArray[p3]).sub(pointArray[p1])
                 );
        normal.normalize();
    }
}
