package JRoff;

public class RoffVector implements Cloneable {

    public double x;
    public double y;
    public double z;

    public RoffVector() {
        x = 0;
        y = 0;
        z = 0;
    }

    public RoffVector(RoffVector rhs) {
       this.x = rhs.x;
       this.y = rhs.y;
       this.z = rhs.z;
    }

    public RoffVector(double x, double y, double z) {
        this.x = x;
        this.y = y;
        this.z = z;
    }

    public RoffVector add(RoffVector rhs) {
        return (new RoffVector(x + rhs.x, y + rhs.y, z + rhs.z));
    }

    public void incBy(RoffVector rhs) {
        this.x += rhs.x;
        this.y += rhs.y;
        this.z += rhs.z;
    }

    public RoffVector sub(RoffVector rhs) {
        return (new RoffVector(x - rhs.x, y - rhs.y, z - rhs.z));
    }

    public void decBy(RoffVector rhs) {
        this.x -= rhs.x;
        this.y -= rhs.y;
        this.z -= rhs.z;
    }

    public RoffVector scalarMultiply(double scalar) {
        return (new RoffVector(x * scalar, y * scalar, z * scalar));
    }

    public void scalarMultiplyBy(double scalar) {
        x *= scalar;
        y *= scalar;
        z *= scalar;
    }

    public RoffVector scalarDivide(double scalar) {
        return (new RoffVector(x / scalar, y / scalar, z / scalar));
    }

    public void scalarDivideBy(double scalar) {
        x /= scalar;
        y /= scalar;
        z /= scalar;
    }

    public RoffVector cross(RoffVector rhs) {
        RoffVector rval = new RoffVector();

	rval.x = (y * rhs.z) - (z * rhs.y);
	rval.y = (z * rhs.x) - (x * rhs.z);
	rval.z = (x * rhs.y) - (y * rhs.x);

        return rval;
    }

    public double getLength() {
        return Math.sqrt((x * x) + (y * y) + (z * z));
    }

    public double getSquaredLength() {
        return ((x * x) + (y * y) + (z * z));
    }

    public double dot(RoffVector rhs) {
	return (x * rhs.x + y * rhs.y + z * rhs.z);
    }

    public void normalize() {
       double length = getLength();
       x = x / length;
       y = y / length;
       z = z / length;
    }

    public void rotateX(double degrees) {
        double radians = degrees * Math.PI / 180.0;
        double cosAngle = Math.cos(radians);
        double sinAngle = Math.sin(radians);
        double origY = y;
        y =	y * cosAngle - z * sinAngle;
        z = origY * sinAngle + z * cosAngle;
    }

    public void rotateY(double degrees) {
        double radians = degrees * Math.PI / 180.0;
        double cosAngle = Math.cos(radians);
        double sinAngle = Math.sin(radians);
        double origX = x;
        x =	x * cosAngle + z * sinAngle;
        z = z * cosAngle - origX * sinAngle;
    }

    public void rotateZ(double degrees) {
        double radians = degrees * Math.PI / 180.0;
        double cosAngle = Math.cos(radians);
        double sinAngle = Math.sin(radians);
        double origX = x;
        x =	x * cosAngle - y * sinAngle;
        y = origX * sinAngle + y * cosAngle;
    }

    public RoffVector rotate(RoffVector axis, double degrees) {
	// Formula goes CCW around axis. I prefer to think in terms of CW
	// rotations, to be consistant with the other rotation methods.
	double radians = -degrees * Math.PI / 180.0;
	double cosAngle = Math.cos(radians);
	double sinAngle = Math.sin(radians);

        RoffVector rval = new RoffVector();
	RoffVector v = this;
	RoffVector w = new RoffVector(axis);
	w.normalize();
	double vDotW = v.dot(w);
	RoffVector vCrossW = v.cross(w);
	w.scalarMultiplyBy(vDotW); // w * (v . w)

	rval.x = w.x + (v.x - w.x) * cosAngle + vCrossW.x * sinAngle;
	rval.y = w.y + (v.y - w.y) * cosAngle + vCrossW.y * sinAngle;
	rval.z = w.z + (v.z - w.z) * cosAngle + vCrossW.z * sinAngle;

        return rval;
    }
}