package JRoff;

import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;

public class RoffViewParameterPanel extends JPanel {

    private JTextField atX;
    private JTextField atY;
    private JTextField atZ;

    private JTextField fromX;
    private JTextField fromY;
    private JTextField fromZ;

    private JTextField upX;
    private JTextField upY;
    private JTextField upZ;

    private JTextField viewAngle;

    public RoffViewParameterPanel(RoffVector at, RoffVector from, RoffVector up, double viewAngle) {

        this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

        atX = new JTextField(getStringValue(at.x), 10);
        atY = new JTextField(getStringValue(at.y), 10);
        atZ = new JTextField(getStringValue(at.z), 10);

        fromX = new JTextField(getStringValue(from.x), 10);
        fromY = new JTextField(getStringValue(from.y), 10);
        fromZ = new JTextField(getStringValue(from.z), 10);

        upX = new JTextField(getStringValue(up.x), 10);
        upY = new JTextField(getStringValue(up.y), 10);
        upZ = new JTextField(getStringValue(up.z), 10);

        this.viewAngle = new JTextField(Double.toString(viewAngle), 5);

        JPanel vecPanel;
        JPanel compPanel;

        vecPanel = new JPanel();
        vecPanel.setLayout(new GridLayout(0, 1));
        vecPanel.setBorder(BorderFactory.createTitledBorder(
                           BorderFactory.createBevelBorder(BevelBorder.LOWERED),
                           "At Point")
                          );

        compPanel = new JPanel();
        compPanel.add(new JLabel("X:"));
        compPanel.add(atX);
        vecPanel.add(compPanel);

        compPanel = new JPanel();
        compPanel.add(new JLabel("Y:"));
        compPanel.add(atY);
        vecPanel.add(compPanel);

        compPanel = new JPanel();
        compPanel.add(new JLabel("Z:"));
        compPanel.add(atZ);
        vecPanel.add(compPanel);
        add(vecPanel);

        vecPanel = new JPanel();
        vecPanel.setLayout(new GridLayout(0, 1));
        vecPanel.setBorder(BorderFactory.createTitledBorder(
                           BorderFactory.createBevelBorder(BevelBorder.LOWERED),
                           "From Point")
                          );

        compPanel = new JPanel();
        compPanel.add(new JLabel("X:"));
        compPanel.add(fromX);
        vecPanel.add(compPanel);

        compPanel = new JPanel();
        compPanel.add(new JLabel("Y:"));
        compPanel.add(fromY);
        vecPanel.add(compPanel);

        compPanel = new JPanel();
        compPanel.add(new JLabel("Z:"));
        compPanel.add(fromZ);
        vecPanel.add(compPanel);
        add(vecPanel);

        vecPanel = new JPanel();
        vecPanel.setLayout(new GridLayout(0, 1));
        vecPanel.setBorder(BorderFactory.createTitledBorder(
                           BorderFactory.createBevelBorder(BevelBorder.LOWERED),
                           "Up Vector")
                          );

        compPanel = new JPanel();
        compPanel.add(new JLabel("X:"));
        compPanel.add(upX);
        vecPanel.add(compPanel);

        compPanel = new JPanel();
        compPanel.add(new JLabel("Y:"));
        compPanel.add(upY);
        vecPanel.add(compPanel);

        compPanel = new JPanel();
        compPanel.add(new JLabel("Z:"));
        compPanel.add(upZ);
        vecPanel.add(compPanel);
        add(vecPanel);

        vecPanel = new JPanel();
        vecPanel.setLayout(new GridLayout(0, 1));
        vecPanel.setBorder(BorderFactory.createTitledBorder(
                           BorderFactory.createBevelBorder(BevelBorder.LOWERED),
                           "View Angle")
                          );
        compPanel = new JPanel();
        compPanel.add(this.viewAngle);
        vecPanel.add(compPanel);
        add(vecPanel);
    }

    public double getAngle() {
        return getDoubleValue(viewAngle);
    }

    public RoffVector getAt() {
        return new RoffVector(getDoubleValue(atX),
                              getDoubleValue(atY),
                              getDoubleValue(atZ));
    }

    public RoffVector getFrom() {
        return new RoffVector(getDoubleValue(fromX),
                              getDoubleValue(fromY),
                              getDoubleValue(fromZ));
    }

    public RoffVector getUp() {
        return new RoffVector(getDoubleValue(upX),
                              getDoubleValue(upY),
                              getDoubleValue(upZ));
    }

    private double getDoubleValue(JTextField inputBox) {

        double rval = 0;
        try {
            rval = Double.parseDouble(inputBox.getText());
        } catch (NumberFormatException e) {
            rval = 0;
        }
        return rval;
    }

    private String getStringValue(double number) {
        String rval = Double.toString(number);
        if (rval.length() > 8) {
            rval = rval.substring(0, 8);
        }
        return rval;
    }
}